package ch.std.meteo.test.service;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import ch.std.meteo.jpa.MeteoData;
import ch.std.meteo.service.MeteoService;
import ch.std.meteo.service.MeteoService.MeteoMetrics;
import ch.std.meteo.test.setup.MeteoDataTestSetup;

@ExtendWith(SpringExtension.class)
@DataJpaTest
@ComponentScan(basePackages = {"ch.std.meteo"})
@EntityScan(basePackages = {"ch.std.meteo"})
@ActiveProfiles("unittest")
public class MeteoServiceTest {
	@Autowired
	private MeteoDataTestSetup meteoDataTestSetup;

	@Autowired
	private MeteoService meteoService;
	
	@BeforeEach
	public void setup() {
		this.meteoDataTestSetup.setup();
	}

	@AfterEach
	public void teardown() {
		this.meteoDataTestSetup.teardown();
	}


	@Test
	public void test_getMeteoDaten() {
		List<MeteoData> meteoDataList = this.meteoService.getMeteoDaten();
		assertNotNull(meteoDataList);
		Double[] loadedTemperatures = meteoDataList.stream().map(md -> md.getTemperature()).toArray(Double[]::new);
		assertArrayEquals(MeteoDataTestSetup.TEMPERATURES, loadedTemperatures);		
	}
	
	@Test
	public void test_getMeteoMetrics() {
		MeteoMetrics meteoMetrics = this.meteoService.getMeteoMetrics();
		assertNotNull(meteoMetrics);
		assertEquals(MeteoDataTestSetup.MIN, meteoMetrics.getMin());
		assertEquals(MeteoDataTestSetup.MAX, meteoMetrics.getMax());	
		assertEquals(MeteoDataTestSetup.MED, meteoMetrics.getMed());	
	}
	
}
