package ch.std.meteo.test.webmvc;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.std.meteo.dto.MeteoDataDTO;
import ch.std.meteo.jpa.MeteoData;
import ch.std.meteo.rest.MeteoController;
import ch.std.meteo.service.MeteoService;
import ch.std.meteo.service.MeteoService.MeteoMetrics;
import ch.std.meteo.test.setup.MeteoDataTestSetup;


@ExtendWith(SpringExtension.class)
@WebMvcTest(MeteoController.class)
public class MeteoServiceWebLayerTest {
	@Autowired
	private MockMvc mvc;

	@MockBean
	private MeteoService meteoService;
	
	@BeforeEach
	public void setup() {
		List<MeteoData> meteoDataList = Arrays.stream(MeteoDataTestSetup.TEMPERATURES).map(t -> new MeteoData(t)).collect(Collectors.toList());
		when(meteoService.getMeteoDaten()).thenReturn(meteoDataList);
		MeteoMetrics meteoMetrics = new MeteoMetrics(meteoDataList);
		when(meteoService.getMeteoMetrics()).thenReturn(meteoMetrics);
	}

	@Test
	public void test_MeteoController_getMeteoDaten() throws Exception {
		MvcResult mvcResult = mvc.perform(get("/rest/data").contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk()).andReturn();
		String actual = mvcResult.getResponse().getContentAsString();
		MeteoDataDTO[] meteoData = new ObjectMapper().readValue(actual, MeteoDataDTO[].class);
		Double[] loadedTemperatures = Arrays.stream(meteoData).map(md -> md.getT()).toArray(Double[]::new);
		assertArrayEquals(MeteoDataTestSetup.TEMPERATURES, loadedTemperatures);		

	}
	
	@Test
	public void test_getMeteoMetrics() throws Exception {
		MvcResult mvcResult = mvc.perform(get("/rest/metrics").contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk()).andReturn();
		String actual = mvcResult.getResponse().getContentAsString();
		MeteoMetrics meteoMetrics = new ObjectMapper().readValue(actual, MeteoMetrics.class);
		assertEquals(MeteoDataTestSetup.MIN, meteoMetrics.getMin());
		assertEquals(MeteoDataTestSetup.MAX, meteoMetrics.getMax());	
		assertEquals(MeteoDataTestSetup.MED, meteoMetrics.getMed());	
	}

}